use embedded_graphics::{
    draw_target::DrawTarget,
    geometry::{Point, Size},
    mono_font::{ascii::FONT_10X20, MonoTextStyleBuilder},
    pixelcolor::{Rgb565, RgbColor},
    primitives::{Primitive, PrimitiveStyleBuilder, Rectangle},
    text::{Baseline, Text},
    Drawable,
};
use heapless::String;

use crate::keyboard::KeyCode;

use super::Element;

const BORDER_COLOR: Rgb565 = Rgb565::new(28, 55, 28);
const BACKGROUND_COLOR: Rgb565 = Rgb565::new(0xFF, 0xFF, 0xFF);
pub const BORDER_WIDTH: u32 = 3;
const CHAR_WIDTH: u32 = 10;
const CHAR_HEIGHT: u32 = 20;

pub struct TextBox {
    // in # of chars
    char_width: u32,
    char_height: u32,
    cursor_x: u32,
    cursor_y: u32,

    cursor_i: usize,

    // in pixels
    x: i32,
    y: i32,

    text: String<32>,
    pub selected: bool,
}

impl TextBox {
    pub fn new(x: i32, y: i32, char_width: u32, char_height: u32) -> Self {
        Self {
            char_width,
            char_height,
            cursor_x: 0,
            cursor_y: 0,
            cursor_i: 0,

            x,
            y,

            text: String::new(),
            selected: true,
        }
    }

    pub fn set_text(&mut self, text: &str) -> Result<(), ()> {
        self.text = text.try_into()?;

        self.cursor_x = u32::try_from(self.text.len()).unwrap() % self.char_width;
        self.cursor_y = u32::try_from(self.text.len()).unwrap() / self.char_width;
        self.cursor_i = self.text.len();

        Ok(())
    }

    pub fn get_text(&self) -> &str {
        &self.text
    }

    fn width(&self) -> u32 {
        BORDER_WIDTH * 2 + CHAR_WIDTH * self.char_width
    }

    fn height(&self) -> u32 {
        BORDER_WIDTH * 2 + CHAR_HEIGHT * self.char_height
    }

    fn cursor_inc_char(&mut self) {
        self.cursor_x += 1;
        if self.cursor_x >= self.char_width {
            self.cursor_x = 0;
            self.cursor_y += 1;
        }
    }

    fn cursor_dec_char(&mut self) {
        if self.cursor_x > 0 {
            self.cursor_x -= 1;
        } else {
            self.cursor_y -= 1;
            self.cursor_x = self.calc_cursor_pos_after_backspace();
        }
    }

    // returns x pos
    fn calc_cursor_pos_after_backspace(&self) -> u32 {
        let mut i = self.cursor_i - 1;
        let mut count = 0;
        while i > 0 {
            i -= 1;
            count += 1;
            // Inefficient; may want to rewrite this if text is laggy
            // O(n)
            if self.text.chars().nth(i).unwrap() == '\n' {
                count -= 1;
                break;
            }
        }

        count % self.char_width
    }
}

impl Element for TextBox {
    fn render<E, DT: DrawTarget<Color = Rgb565, Error = E>>(
        &mut self,
        dt: &mut DT,
    ) -> Result<(), E> {
        // Draw border/background
        let rect_style = PrimitiveStyleBuilder::new()
            .stroke_color(BORDER_COLOR)
            .stroke_width(BORDER_WIDTH)
            .fill_color(BACKGROUND_COLOR)
            .build();
        Rectangle::new(
            Point::new(self.x, self.y),
            Size::new(self.width(), self.height()),
        )
        .into_styled(rect_style)
        .draw(dt)?;

        // Draw text
        let text_x = self.x + i32::try_from(BORDER_WIDTH).unwrap();
        let text_y = self.y + i32::try_from(BORDER_WIDTH).unwrap();

        let text_style = MonoTextStyleBuilder::new()
            .font(&FONT_10X20)
            .text_color(Rgb565::BLACK)
            .build();

        let mut cur_x = 0;
        let mut cur_y = 0;
        let mut buf = [0; 4];
        for c in self.text.chars() {
            if c == '\n' {
                cur_x = 0;
                cur_y += 1;

                continue;
            }

            Text::with_baseline(
                c.encode_utf8(&mut buf),
                Point::new(
                    text_x + i32::try_from(cur_x * CHAR_WIDTH).unwrap(),
                    text_y + i32::try_from(cur_y * CHAR_HEIGHT).unwrap(),
                ),
                text_style,
                Baseline::Top,
            )
            .draw(dt)?;

            cur_x += 1;
            if cur_x == self.char_width {
                cur_x = 0;
                cur_y += 1;
            }
        }

        // Draw cursor
        if self.selected {
            Text::with_baseline(
                "_",
                Point::new(
                    text_x + i32::try_from(self.cursor_x * CHAR_WIDTH).unwrap(),
                    text_y + i32::try_from(self.cursor_y * CHAR_HEIGHT).unwrap(),
                ),
                text_style,
                Baseline::Top,
            )
            .draw(dt)?;
        }

        Ok(())
    }

    fn key_push(&mut self, k: KeyCode) {
        match k {
            KeyCode::Backspace => {
                if let Some(popped) = self.text.pop() {
                    if popped == '\n' {
                        self.cursor_x = self.calc_cursor_pos_after_backspace();
                        self.cursor_y -= 1;
                    } else {
                        self.cursor_dec_char();
                    }
                    self.cursor_i -= 1;
                }
            }
            KeyCode::Char(c) => {
                let _ = self.text.push(c);
                self.cursor_inc_char();
                self.cursor_i += 1;
            }
            KeyCode::Enter => {
                let _ = self.text.push('\n');
                self.cursor_x = 0;
                self.cursor_y += 1;
                self.cursor_i += 1;
            }
            _ => {}
        }
    }
}
